@@ -345,8 +345,15 @@ get_one_option(const struct my_option *opt, const char *argument,
   case 'S':
     if (filename[0] == '\0')
     {
-      /* Socket given on command line, switch protocol to use SOCKETSt */
-      opt_protocol= MYSQL_PROTOCOL_SOCKET;
+      /*
+        Socket given on command line, switch protocol to use SOCKETSt
+        Except on Windows if 'protocol= pipe' has been provided in
+        the config file or command line.
+      */
+      if (opt_protocol != MYSQL_PROTOCOL_PIPE)
+      {
+        opt_protocol= MYSQL_PROTOCOL_SOCKET;
+      }
     }
     break;
     break;
@@ -377,7 +384,7 @@ get_options(int *argc,char ***argv)
     exit(ho_error);
   
   if (tty_password)
-    opt_password=get_tty_password(NullS);
+    opt_password=my_get_tty_password(NullS);
   if (opt_count)
   {
     /*
